@@ -2,10 +2,13 @@
 {-# LANGUAGE GADTs #-}
 {-# LANGUAGE Rank2Types #-}
 {-# LANGUAGE TypeFamilies #-}
+#if __GLASGOW_HASKELL__ >= 706
+{-# LANGUAGE PolyKinds #-}
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Equality
--- Copyright   :  (C) 2012-15 Edward Kmett
+-- Copyright   :  (C) 2012-16 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -29,7 +32,7 @@ module Control.Lens.Equality
   ) where
 
 import Control.Lens.Type
-import Data.Functor.Identity
+import Data.Proxy (Proxy)
 
 #ifdef HLINT
 {-# ANN module "HLint: ignore Use id" #-}
@@ -48,7 +51,11 @@ data Identical a b s t where
   Identical :: Identical a b a b
 
 -- | When you see this as an argument to a function, it expects an 'Equality'.
-type AnEquality s t a b = Identical a (Identity b) a (Identity b) -> Identical a (Identity b) s (Identity t)
+#if __GLASGOW_HASKELL__ >= 706
+type AnEquality (s :: k1) (t :: k2) (a :: k1) (b :: k2) = Identical a (Proxy b) a (Proxy b) -> Identical a (Proxy b) s (Proxy t)
+#else
+type AnEquality s t a b = Identical a (Proxy b) a (Proxy b) -> Identical a (Proxy b) s (Proxy t)
+#endif
 
 -- | A 'Simple' 'AnEquality'.
 type AnEquality' s a = AnEquality s s a a
@@ -65,7 +72,11 @@ substEq l = case runEq l of
 {-# INLINE substEq #-}
 
 -- | We can use 'Equality' to do substitution into anything.
+#if __GLASGOW_HASKELL__ >= 706
+mapEq :: forall (s :: k1) (t :: k2) (a :: k1) (b :: k2) (f :: k1 -> *) . AnEquality s t a b -> f s -> f a
+#else
 mapEq :: AnEquality s t a b -> f s -> f a
+#endif
 mapEq l r = substEq l r
 {-# INLINE mapEq #-}
 
