@@ -5,7 +5,7 @@
 ------------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Reified
--- Copyright   :  (C) 2012-15 Edward Kmett
+-- Copyright   :  (C) 2012-16 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -33,6 +33,7 @@ import Data.Functor.Bind
 import Data.Functor.Extend
 import Data.Functor.Identity
 import Data.Functor.Plus
+import Data.Profunctor.Closed
 import Data.Profunctor
 import Data.Profunctor.Rep
 import Data.Profunctor.Sieve
@@ -150,7 +151,7 @@ instance Bind (ReifiedGetter s) where
   {-# INLINE (>>-) #-}
 
 instance Monad (ReifiedGetter s) where
-  return a = Getter $ to $ \_ -> a
+  return = pure
   {-# INLINE return #-}
   Getter ma >>= f = Getter $ to $ \s -> view (runGetter (f (view ma s))) s
   {-# INLINE (>>=) #-}
@@ -169,6 +170,9 @@ instance Profunctor ReifiedGetter where
   rmap f l    = Getter $ runGetter l.to f
   {-# INLINE rmap #-}
 
+instance Closed ReifiedGetter where
+  closed l = Getter $ to $ \f -> view (runGetter l) . f
+
 instance Cosieve ReifiedGetter Identity where
   cosieve (Getter l) = view l . runIdentity
 
@@ -276,6 +280,7 @@ instance Semigroup i => Apply (ReifiedIndexedGetter i s) where
         (j, a) -> phantom $ indexed k (i <> j) (f a)
   {-# INLINE (<.>) #-}
 
+
 ------------------------------------------------------------------------------
 -- Fold
 ------------------------------------------------------------------------------
@@ -383,7 +388,7 @@ instance Bind (ReifiedFold s) where
   {-# INLINE (>>-) #-}
 
 instance Monad (ReifiedFold s) where
-  return a = Fold $ folding $ \_ -> [a]
+  return = pure
   {-# INLINE return #-}
   Fold ma >>= f = Fold $ folding $ \s -> toListOf ma s >>= \a -> toListOf (runFold (f a)) s
   {-# INLINE (>>=) #-}
@@ -472,6 +477,7 @@ instance Strong (ReifiedIndexedFold i) where
     phantom $ runIndexedFold l (dimap ((,) c) phantom f) s
   {-# INLINE second' #-}
 
+
 ------------------------------------------------------------------------------
 -- Setter
 ------------------------------------------------------------------------------
