@@ -7,6 +7,7 @@
 {-# LANGUAGE FunctionalDependencies #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE Trustworthy #-}
+{-# LANGUAGE ConstraintKinds #-}
 
 #ifndef MIN_VERSION_containers
 #define MIN_VERSION_containers(x,y,z) 1
@@ -80,7 +81,7 @@ module Control.Lens.Traversal
   -- * Common Traversals
   , Traversable(traverse)
   , Traversable1(traverse1)
-  , both
+  , both, both1
   , beside
   , taking
   , dropping
@@ -144,6 +145,7 @@ import Control.Lens.Type
 import Control.Monad
 import Control.Monad.Trans.State.Lazy
 import Data.Bitraversable
+import Data.CallStack
 #if __GLASGOW_HASKELL__ < 710
 import Data.Foldable (Foldable)
 #endif
@@ -163,6 +165,7 @@ import Data.Profunctor.Sieve
 import Data.Profunctor.Unsafe
 import Data.Reflection
 import Data.Semigroup.Traversable
+import Data.Semigroup.Bitraversable
 import Data.Tagged
 import Data.Traversable
 import Data.Tuple (swap)
@@ -271,9 +274,10 @@ type Traversing1' p f s a = Traversing1 p f s s a a
 -- @
 --
 -- @
--- 'traverseOf' :: 'Functor' f => 'Iso' s t a b       -> (a -> f b) -> s -> f t
--- 'traverseOf' :: 'Functor' f => 'Lens' s t a b      -> (a -> f b) -> s -> f t
--- 'traverseOf' :: 'Applicative' f => 'Traversal' s t a b -> (a -> f b) -> s -> f t
+-- 'traverseOf' :: 'Functor' f     => 'Iso' s t a b        -> (a -> f b) -> s -> f t
+-- 'traverseOf' :: 'Functor' f     => 'Lens' s t a b       -> (a -> f b) -> s -> f t
+-- 'traverseOf' :: 'Apply' f       => 'Traversal1' s t a b -> (a -> f b) -> s -> f t
+-- 'traverseOf' :: 'Applicative' f => 'Traversal' s t a b  -> (a -> f b) -> s -> f t
 -- @
 traverseOf :: LensLike f s t a b -> (a -> f b) -> s -> f t
 traverseOf = id
@@ -654,7 +658,7 @@ holesOf l s = unTagged
 -- 'singular' :: 'IndexedTraversal' i s t a a -> 'IndexedLens' i s t a a
 -- 'singular' :: 'IndexedFold' i s a          -> 'IndexedGetter' i s a
 -- @
-singular :: (Conjoined p, Functor f)
+singular :: (HasCallStack, Conjoined p, Functor f)
          => Traversing p f s t a a
          -> Over p f s t a a
 singular l = conjoined
@@ -680,7 +684,7 @@ singular l = conjoined
 -- 'unsafeSingular' :: 'IndexedTraversal' i s t a b -> 'IndexedLens' i s t a b
 -- 'unsafeSingular' :: 'IndexedFold' i s a          -> 'IndexedGetter' i s a
 -- @
-unsafeSingular :: (Conjoined p, Functor f)
+unsafeSingular :: (HasCallStack, Conjoined p, Functor f)
                => Traversing p f s t a b
                -> Over p f s t a b
 unsafeSingular l = conjoined
@@ -753,6 +757,18 @@ both :: Bitraversable r => Traversal (r a a) (r b b) a b
 both f = bitraverse f f
 {-# INLINE both #-}
 
+-- | Traverse both parts of a 'Bitraversable1' container with matching types.
+--
+-- Usually that type will be a pair.
+--
+-- @
+-- 'both1' :: 'Traversal1' (a, a)       (b, b)       a b
+-- 'both1' :: 'Traversal1' ('Either' a a) ('Either' b b) a b
+-- @
+both1 :: Bitraversable1 r => Traversal1 (r a a) (r b b) a b
+both1 f = bitraverse1 f f
+{-# INLINE both1 #-}
+
 -- | Apply a different 'Traversal' or 'Fold' to each side of a 'Bitraversable' container.
 --
 -- @
@@ -1256,11 +1272,7 @@ infixl 5 `failing`
 -- 'deepOf' :: 'Traversal' s t s t -> 'IndexedTraversal' i s t a b -> 'IndexedTraversal' i s t a b
 -- @
 deepOf :: (Conjoined p, Applicative f) => LensLike f s t s t -> Traversing p f s t a b -> Over p f s t a b
-deepOf r l pafb = go
-  where go s = case pins b of
-          [] -> r go s
-          xs -> unsafeOuts b <$> traverse (cosieve pafb) xs
-          where b = l sell s
+deepOf r l = failing l (r . deepOf r l)
 
 -- | "Fuse" a 'Traversal' by reassociating all of the '\<*\>' operations to the
 -- left and fusing all of the 'fmap' calls into one. This is particularly
