@@ -65,7 +65,6 @@ module Data.Map.Lens
   ) where
 
 import Control.Lens.Getter ( IndexedGetting, iviews )
-import Control.Lens.Type
 import qualified Data.Map as Map
 
 -- $setup
@@ -81,6 +80,9 @@ import qualified Data.Map as Map
 -- >>> toMapOf folded ["hello", "world"]
 -- fromList [(0,"hello"),(1,"world")]
 --
+-- >>> toMapOf (folded . ifolded) [('a',"alpha"),('b', "beta")]
+-- fromList [('a',"alpha"),('b',"beta")]
+--
 -- >>> toMapOf (folded <.> folded) ["foo", "bar"]
 -- fromList [((0,0),'f'),((0,1),'o'),((0,2),'o'),((1,0),'b'),((1,1),'a'),((1,2),'r')]
 --
